=== modified file 'drizzled/execute.cc'
--- drizzled/execute.cc	2011-04-14 02:59:27 +0000
+++ drizzled/execute.cc	2011-04-26 02:27:51 +0000
@@ -64,6 +64,8 @@
     // session. Eventually we will allow someone to change the effective
     // user.
     new_session->user()= _session.user();
+    new_session->setOriginatingServerUUID(_session.getOriginatingServerUUID());
+    new_session->setOriginatingCommitID(_session.getOriginatingCommitID());
     
     if (Session::schedule(new_session))
     {

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2011-04-22 15:01:43 +0000
+++ drizzled/session.cc	2011-04-26 02:27:51 +0000
@@ -240,6 +240,7 @@
   tablespace_op(false),
   use_usage(false),
   security_ctx(identifier::User::make_shared()),
+  originating_server_uuid_set(false),
   client(client_arg)
 {
   client->setSession(this);
@@ -255,6 +256,8 @@
   lex().current_select= 0;
   memset(&variables, 0, sizeof(variables));
   scoreboard_index= -1;
+  originating_server_uuid= "";
+  originating_commit_id= 0;
   cleanup_done= abort_on_warning= no_warnings_for_error= false;
 
   /* query_cache init */

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2011-04-25 02:02:23 +0000
+++ drizzled/session.h	2011-04-26 02:27:51 +0000
@@ -258,6 +258,32 @@
     scoreboard_index= in_scoreboard_index;
   }
 
+  bool isOriginatingServerUUIDSet()
+  {
+    return originating_server_uuid_set;
+  }
+
+  void setOriginatingServerUUID(std::string in_originating_server_uuid)
+  {
+    originating_server_uuid= in_originating_server_uuid;
+    originating_server_uuid_set= true;
+  }
+
+  std::string &getOriginatingServerUUID()
+  {
+    return originating_server_uuid;
+  }
+
+  void setOriginatingCommitID(uint64_t in_originating_commit_id)
+  {
+    originating_commit_id= in_originating_commit_id;
+  }
+
+  uint64_t getOriginatingCommitID()
+  {
+    return originating_commit_id;
+  }
+
   /**
    * Is this session viewable by the current user?
    */
@@ -1168,6 +1194,9 @@
   rusage usage;
   identifier::user::mptr security_ctx;
   int32_t scoreboard_index;
+  bool originating_server_uuid_set;
+  std::string originating_server_uuid;
+  uint64_t originating_commit_id;
   plugin::Client *client;
 };
 

=== modified file 'plugin/innobase/dict/create_replication.cc'
--- plugin/innobase/dict/create_replication.cc	2011-04-04 21:37:55 +0000
+++ plugin/innobase/dict/create_replication.cc	2011-04-26 02:27:51 +0000
@@ -77,7 +77,7 @@
   error = que_eval_sql(info,
                        "PROCEDURE CREATE_SYS_REPLICATION_LOG_PROC () IS\n"
                        "BEGIN\n"
-                       "CREATE TABLE SYS_REPLICATION_LOG(ID INT(8), SEGID INT, COMMIT_ID INT(8), END_TIMESTAMP INT(8), MESSAGE_LEN INT, MESSAGE BLOB);\n" 
+                       "CREATE TABLE SYS_REPLICATION_LOG(ID INT(8), SEGID INT, COMMIT_ID INT(8), END_TIMESTAMP INT(8), ORIGINATING_SERVER_UUID BLOB, ORIGINATING_COMMIT_ID INT(8), MESSAGE_LEN INT, MESSAGE BLOB);\n" 
                        "CREATE UNIQUE CLUSTERED INDEX PRIMARY ON SYS_REPLICATION_LOG (ID, SEGID);\n"
                        "CREATE INDEX COMMIT_IDX ON SYS_REPLICATION_LOG (COMMIT_ID, ID);\n"
                        "END;\n"
@@ -148,6 +148,14 @@
   field->set_type(drizzled::message::Table::Field::BIGINT);
 
   field= table_message->add_field();
+  field->set_name("ORIGINATING_SERVER_UUID");
+  field->set_type(drizzled::message::Table::Field::BLOB);
+
+  field= table_message->add_field();
+  field->set_name("ORIGINATING_COMMIT_ID");
+  field->set_type(drizzled::message::Table::Field::BIGINT);
+
+  field= table_message->add_field();
   field->set_name("MESSAGE_LEN");
   field->set_type(drizzled::message::Table::Field::INTEGER);
 
@@ -192,7 +200,10 @@
 ulint insert_replication_message(const char *message, size_t size, 
                                  trx_t *trx, uint64_t trx_id, 
                                  uint64_t end_timestamp, bool is_end_segment, 
-                                 uint32_t seg_id) 
+                                 uint32_t seg_id, const char *server_uuid,
+                                 bool use_originating_server_uuid,
+                                 const char *originating_server_uuid,
+                                 uint64_t originating_commit_id)
 {
   ulint error;
   row_prebuilt_t*	prebuilt;	/* For reading rows */
@@ -252,12 +263,28 @@
   row_mysql_store_col_in_innobase_format(dfield, data, TRUE, (byte*)&end_timestamp, 8, dict_table_is_comp(prebuilt->table));
   dfield_set_data(dfield, data, 8);
 
+  if (not use_originating_server_uuid)
+  {
+    /* This transaction originated from this server, rather then being
+       replicated to this server reset the values to reflect that */
+    originating_server_uuid= server_uuid;
+    originating_commit_id= commit_id;
+  }
+
   dfield = dtuple_get_nth_field(dtuple, 4);
+  dfield_set_data(dfield, originating_server_uuid, 37);
+
+  dfield = dtuple_get_nth_field(dtuple, 5);
+  data= static_cast<byte*>(mem_heap_alloc(prebuilt->heap, 8));
+  row_mysql_store_col_in_innobase_format(dfield, data, TRUE, (byte*)&originating_commit_id, 8, dict_table_is_comp(prebuilt->table));
+  dfield_set_data(dfield, data, 8);
+
+  dfield = dtuple_get_nth_field(dtuple, 6);
   data= static_cast<byte*>(mem_heap_alloc(prebuilt->heap, 4));
   row_mysql_store_col_in_innobase_format(dfield, data, TRUE, (byte*)&size, 4, dict_table_is_comp(prebuilt->table));
   dfield_set_data(dfield, data, 4);
 
-  dfield = dtuple_get_nth_field(dtuple, 5);
+  dfield = dtuple_get_nth_field(dtuple, 7);
   dfield_set_data(dfield, message, size);
 
   ins_node_t*	node		= prebuilt->ins_node;
@@ -356,8 +383,16 @@
     convert_to_mysql_format(timestampbyte, field, 8);
     ret.end_timestamp= *(uint64_t *)timestampbyte;
 
+    field = rec_get_nth_field_old(rec, 6, &len);
+    ret.originating_server_uuid= (char *)field;
+
+    field = rec_get_nth_field_old(rec, 7, &len);
+    byte originatingcommitbyte[8];
+    convert_to_mysql_format(originatingcommitbyte, field, 8);
+    ret.originating_commit_id= *(uint64_t *)originatingcommitbyte;
+
     // Handler message
-    field = rec_get_nth_field_old(rec, 7, &len);
+    field = rec_get_nth_field_old(rec, 9, &len);
     ret.message= (char *)field;
     ret.message_length= len;
 

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2011-04-15 14:40:55 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2011-04-26 02:27:51 +0000
@@ -1019,8 +1019,17 @@
   uint64_t end_timestamp= message.transaction_context().end_timestamp();
   bool is_end_segment= message.end_segment();
   trx->log_commit_id= TRUE;
+
+  string server_uuid= session.getServerUUID();
+  string originating_server_uuid= session.getOriginatingServerUUID();
+  uint64_t originating_commit_id= session.getOriginatingCommitID();
+  bool use_originating_server_uuid= session.isOriginatingServerUUIDSet();
+
   ulint error= insert_replication_message(data, message.ByteSize(), trx, trx_id,
-               end_timestamp, is_end_segment, seg_id);
+               end_timestamp, is_end_segment, seg_id, server_uuid.c_str(),
+               use_originating_server_uuid, originating_server_uuid.c_str(),
+               originating_commit_id);
+
   (void)error;
 
   delete[] data;

=== modified file 'plugin/innobase/handler/replication_dictionary.cc'
--- plugin/innobase/handler/replication_dictionary.cc	2011-02-17 00:14:13 +0000
+++ plugin/innobase/handler/replication_dictionary.cc	2011-04-26 02:27:51 +0000
@@ -79,6 +79,8 @@
   add_field("TRANSACTION_SEGMENT_ID", plugin::TableFunction::NUMBER, 0, false);
   add_field("COMMIT_ID", plugin::TableFunction::NUMBER, 0, false);
   add_field("END_TIMESTAMP", plugin::TableFunction::NUMBER, 0, false);
+  add_field("ORIGINATING_SERVER_UUID", plugin::TableFunction::STRING, 37, false);
+  add_field("ORIGINATING_COMMIT_ID", plugin::TableFunction::NUMBER, 0, false);
   add_field("TRANSACTION_MESSAGE_STRING", plugin::TableFunction::STRING, transaction_message_threshold, false);
   add_field("TRANSACTION_LENGTH", plugin::TableFunction::NUMBER, 0, false);
 }
@@ -110,6 +112,10 @@
   push(static_cast<uint64_t>(ret.commit_id));
 
   push(static_cast<uint64_t>(ret.end_timestamp));
+
+  push(ret.originating_server_uuid);
+
+  push(static_cast<uint64_t>(ret.originating_commit_id));
   
   /* Message in viewable format */
   bool result= message.ParseFromArray(ret.message, ret.message_length);

=== modified file 'plugin/innobase/include/create_replication.h'
--- plugin/innobase/include/create_replication.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/create_replication.h	2011-04-26 02:27:51 +0000
@@ -45,10 +45,13 @@
 
 UNIV_INTERN ulint dict_create_sys_replication_log(void);
 
-UNIV_INTERN ulint insert_replication_message(const char *message, size_t size, 
-                                             trx_t *trx, uint64_t trx_id, 
+UNIV_INTERN ulint insert_replication_message(const char *message, size_t size,
+                                             trx_t *trx, uint64_t trx_id,
                                              uint64_t end_timestamp, bool is_end_segment,
-                                             uint32_t seg_id);
+                                             uint32_t seg_id, const char *server_uuid,
+                                             bool use_originating_server_uuid,
+                                             const char *originating_server_uuid,
+                                             uint64_t originating_commit_id);
 
 UNIV_INTERN struct read_replication_state_st *replication_read_init(void);
 UNIV_INTERN void replication_read_deinit(struct read_replication_state_st *);

=== modified file 'plugin/innobase/include/read_replication.h'
--- plugin/innobase/include/read_replication.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/read_replication.h	2011-04-26 02:27:51 +0000
@@ -27,6 +27,8 @@
   unsigned long long commit_id;
   unsigned long long end_timestamp;
   unsigned long seg_id;
+  const char *originating_server_uuid;
+  unsigned long long originating_commit_id;
   unsigned long long message_length;
   const char *message;
 };

